/*-------------------------------------------------------------------
 * fixed_order2butterworth10 -- 2nd order BW 10Hz - fixed point
 *-------------------------------------------------------------------
 */
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

int     n;
#define MAXLEN	1000
#define MAXFLD	10

#define WINLEN	20

int	time;
double	sample;
double	x[WINLEN], y[WINLEN];
double	sum;
double	average;

int     getargs(char *cmd, char *argv[]);


/*----------------------------------------------------------------
 * float2fix -- convert float to fixed point 18+14 bits
 *----------------------------------------------------------------
 */
int     float2fix(double x)
{
	int	y;

	y = x * (1 << 14);
	return y;
}


/*----------------------------------------------------------------
 * fix2float -- convert fixed 18+14 bits to float
 *----------------------------------------------------------------
 */
double 	fix2float(int x)
{
	double	y;

	y = ((double) x) / (1 << 14);
	return y;
}


/*----------------------------------------------------------------
 * fixmul -- multiply fixed 18+14 bits to float
 *----------------------------------------------------------------
 */
double 	fixmul(int x1, int x2)
{
	int	y;

	y = x1 * x2; // Note: be sure this fits in 32 bits !!!!
	y = (y >> 14);
	return y;
}


/*----------------------------------------------------------------
 * main
 *----------------------------------------------------------------
 */
int     main(int argc, char **argv)
{
	int	N, i, fieldnum, indexnum, maxfieldnum;
	char	s[MAXLEN], u[MAXLEN];
        char    *field[MAXFLD];

	if (argc != 1) {
		fprintf(stderr,"usage: %s\n",argv[0]);
		exit(1);
	}

        /* initialize history
         */
	for (n = 0; n < WINLEN; n++) {
		x[n] = 0;
		y[n] = 0;
	}

        /* read and process data
         */
        n = 0;
	average = 0;
	maxfieldnum = 0;
        for (;;) {

		/* read x
		 */
                fgets(s,MAXLEN,stdin);
                if (feof(stdin))
                        break;

		s[strlen(s)-1] = '\0';
		strcpy(u,s);
		fieldnum = getargs(u,field);

		if (fieldnum != 2) {
			fprintf(stderr,"must have 2 fields\n");
			exit(-1);
		}
		// printf("%s %s\n",field[0],field[1]);
		time = atoi(field[0]);
		sample = atof(field[1]);

		/* update sample history
		 */
		for (i = WINLEN-1; i > 0; i--) {
			x[i] = x[i-1];
			y[i] = y[i-1];
		}
		x[0] = sample;


		/* Actual filter section:
		 * ======================
		 *
		 * convert to fixed point
		 * do the calculations in fixed point
		 * convert back to floating point
		 */

		/* for 2rd order 10Hz LP @ 200Hz sample rate
		 * the coeeficients in floating point are
		 * a[0] = 0.020083372;
		 * a[1] = 0.040166743;
		 * a[2] = 0.020083372;
		 * b[0] = 1.0;
		 * b[1] = -1.5610181;
		 * b[2] = 0.6413515;
		 */

		/* in fixed point (32 bits, 14 bits fraction):
		 */
		int a0 = float2fix(0.020083372); 
		int a1 = float2fix(0.040166743); 
		int a2 = float2fix(0.020083372); 
		int b1 = float2fix(-1.5610181); 
		int b2 = float2fix(0.6413515); 

		int x0 = float2fix(x[0]);
		int x1 = float2fix(x[1]);
		int x2 = float2fix(x[2]);
		int y1 = float2fix(y[1]);
		int y2 = float2fix(y[2]);

		int y0 = fixmul(a0,x0)+fixmul(a1,x1)+fixmul(a2,x2)-
		     fixmul(b1,y1)-fixmul(b2,y2);
		y[0] = fix2float(y0);

		/* write y
		 */
		printf("%d %f\n",time,y[0]);

                n++;
        }
	fprintf(stderr,"%d samples processed\n",n);
        fclose(stdin);
}


/*----------------------------------------------------------------
 * getargs -- parse command line in argc / argv format
 *----------------------------------------------------------------
 */
int     getargs(char *cmd, char *argv[])
{
        int             argc;
        int             c,l;

        l = strlen(cmd);
        c = 0;
        for (argc = 0; c < l; argc++) {
                while (c < l && (cmd[c] == ' ' || cmd[c] == '\t'))
                        c++;
                if (c == l)
                        break;
                argv[argc] = &(cmd[c]);
                while (c < l && (cmd[c] != ' ' && cmd[c] != '\t'))
                        c++;
                cmd[c] = '\0';
                c++;
        }
        return argc;
}

