/*------------------------------------------------------------------------
 *
 * order2_butterworth10.c -- IIR filter Butterworth 2nd order 10Hz (5 ms)
 *
 *------------------------------------------------------------------------
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>


int     k;
#define MAXLEN	1000
#define MAXFLD	10



#define MAXWIN	10
int	t;
float	sample;
float	x[MAXWIN];
float	y[MAXWIN];
float	temp;

float	a[10];
float	b[10];

int     main(argc,argv)
int     argc;
char    **argv;
{
	int	i, fieldnum, indexnum, maxfieldnum;
	char	s[MAXLEN], u[MAXLEN];
        char    *field[MAXFLD];

	if (argc != 1) {
		fprintf(stderr,"%s: no args allowed\n",argv[0]);
		exit(1);
	}

        /* initialize coefficient arrays
         */
	for (i = 0; i < MAXWIN; i++) {
		a[i] = 0;
		b[i] = 0;
	}

        /* fill in coefficients
         */

	/* 2rd order 10Hz LP @ 200Hz sample rate:
	 */
	a[0] = 0.020083372;
	a[1] = 0.040166743;
	a[2] = 0.020083372;
	b[0] = 1.0;
	b[1] = -1.5610181;
	b[2] = 0.6413515;


        /* initialize x, y history
         */
	for (i = 0; i < MAXWIN; i++) {
		x[i] = 0;
	}

        /* read and process data
         */
        k = 0;
	maxfieldnum = 0;
        for (;;) {

		/* read x
		 */
                fgets(s,MAXLEN,stdin);
                if (feof(stdin))
                        break;

		s[strlen(s)-1] = '\0';
		strcpy(u,s);
		fieldnum = getargs(u,field);

		if (fieldnum != 2) {
			fprintf(stderr,"must have 2 fields\n");
			exit(-1);
		}
		// printf("%s %s\n",field[0],field[1]);
		t = atoi(field[0]);
		sample = atof(field[1]);

		/* update x and y history
		 */
		for (i = MAXWIN-1; i > 0; i--) {
			x[i] = x[i-1];
			y[i] = y[i-1];
		}
		x[0] = sample;
		// y[0] is to be computed by IIR

		/* IIR filter 
		 * b[0] * y[0] + b[1] * y[1] .. = a[0] * x[0] + a[1] * x[1] ..
		 */
		y[0] = a[0] * x[0];
		/*
		printf("y[0] = a[0] * x[0]: %e = %e * %e\n",
			y[0],a[0],x[0]);
		*/
		for (i = 1; i < MAXWIN; i++) {
			temp = y[0];
			y[0] = y[0] + a[i] * x[i] - b[i] * y[i];
			/*
			printf("y[0] = y[0] + a[%d] * x[%d] - b[%d] * y[%d]: ",
				i,i,i,i);
			printf("%e = %e + %e * %e - %e * %e\n",
				y[0],temp,a[i],x[i],b[i],y[i]);
			*/
		}
		temp = y[0];
		y[0] = y[0] / b[0];
		// printf("y[0] = y[0] / b[0]: %e = %e / %e\n",y[0],temp,b[0]);
		

		/* write y
		 */
		printf("%d %f\n",t,y[0]);

                k++;
        }
	fprintf(stderr,"%d samples processed\n",k);
        fclose(stdin);
}


/*----------------------------------------------------------------
 * getargs -- parse command line in argc / argv format
 *----------------------------------------------------------------
 */
int             getargs(cmd,argv)
char            *cmd,*argv[];
{
        int             argc;
        int             c,l;

        l = strlen(cmd);
        c = 0;
        for (argc = 0; c < l; argc++) {
                while (c < l && (cmd[c] == ' ' || cmd[c] == '\t'))
                        c++;
                if (c == l)
                        break;
                argv[argc] = &(cmd[c]);
                while (c < l && (cmd[c] != ' ' && cmd[c] != '\t'))
                        c++;
                cmd[c] = '\0';
                c++;
        }
        return argc;
}

